`timescale 1ns / 1ps
//
// basic serial protocol IO device driver, but on transmit sends 2 bytes, LSB first
//
// CLKS_BER_BIT = ratio of internal clock to baud rate desired
// o_debug: lower 7 bits come from RX, upper from TX module
//          see uart_* for what bits are where

module SerialIO2 #(parameter CLKS_PER_BIT = 'd50) (
    input i_Clock,
    input i_Reset,
    output o_Tx,
    input i_Rx,
    input i_Transmit,
    input [15:0] i_Tx_Byte2,
    output o_Tx_Active,
    output reg o_Tx_Done,
    output [7:0] o_Rx_Byte,
    output o_Rx_DV,
    output [15:0] o_debug

    );

//
// when i_Transmit is asserted, send 2 bytes
// we will add in a pause state.  if the clock is 20ns, and we are running
// at 1Mbaud, or 1us, then we need 50 clock ticks for the pause equivalent
// to a single bit.  to be safe lets use 8 bits pause, to give the receiver
// time to breath, so that's 400 clock ticks, so if we use 8 bits we should
// be safe!
//
reg Tx_send;
wire tx1_done;
reg[7:0] tx_byte1;
reg[7:0] tx_byte2;
reg [7:0] Tx_Byte;
reg [2:0] tx_state;
reg [7:0] pause_counter;
wire pause_done = (pause_counter == 8'hFF);
localparam [2:0] WAIT=0, SEND1=1, WAIT_DONE1=2, PAUSE=3, SEND2=4, WAIT_DONE2=5, WAIT_DONE=6;
always @ (posedge i_Clock) 
    if (i_Reset) begin
        o_Tx_Done <= 0;
        Tx_send <= 0;
        tx_state <= WAIT;
        tx_byte1 <= 'h0;
        tx_byte2 <= 'h0;
        pause_counter <= 'h00;
    end
    else case (tx_state)
        WAIT: begin
            o_Tx_Done <= 0;
            pause_counter <= 'h00;
            tx_byte1 <= i_Tx_Byte2[7:0];
            Tx_Byte <= i_Tx_Byte2[7:0];
            tx_byte2 <= i_Tx_Byte2[15:8];
            Tx_send <= 0;
            if (i_Transmit) tx_state <= SEND1;
            else tx_state <= WAIT;
        end
        SEND1: begin
            Tx_send <= 1;
            tx_state <= WAIT_DONE1;
        end
        WAIT_DONE1: begin
            Tx_send <= 0;
            if (tx1_done) tx_state <= PAUSE;
            else tx_state <= WAIT_DONE1;
        end 
        PAUSE: begin
            Tx_Byte <= i_Tx_Byte2[15:8];
            pause_counter <= pause_counter + 1;
            if (pause_done) tx_state <= SEND2;
            else tx_state <= PAUSE;
        end
        SEND2: begin
            Tx_send <= 1;
            tx_state <= WAIT_DONE2;
        end
        WAIT_DONE2: begin
            Tx_send <= 0;
            if (tx1_done) tx_state <= WAIT_DONE;
            else tx_state <= WAIT_DONE2;
        end
        WAIT_DONE: begin
            o_Tx_Done <= 1;
            if (i_Transmit) tx_state <= WAIT_DONE;
            else tx_state <= WAIT;
        end
        default: begin
            o_Tx_Done <= 0;
            pause_counter <= 'h00;
            tx_state <= WAIT;
            tx_byte1 <= 'h0;
            tx_byte2 <= 'h0;
            pause_counter <= 'h00;
            Tx_send <= 0;
        end 
    endcase
wire [15:0] clocks_per_bit = CLKS_PER_BIT;
wire [7:0] tdebug;
uart_tx TX (
    .i_Clocks_per_Bit(clocks_per_bit),
    .i_Clock(i_Clock),
    .i_Reset(i_Reset),
    .i_Tx_DV(Tx_send),
    .i_Tx_Byte(Tx_Byte),
    .o_Tx_Serial(o_Tx),
    .o_Tx_Active(o_Tx_Active),
    .o_Tx_Done(tx1_done),
    .o_debug(tdebug)
    );

wire [7:0] rdebug;
uart_rx RX  (
    .i_Clocks_per_Bit(clocks_per_bit),
    .i_Clock(i_Clock),
    .i_Reset(i_Reset),
    .i_Rx_Serial(i_Rx),
    .o_Rx_Byte(o_Rx_Byte),
    .o_Rx_DV(o_Rx_DV),
    .o_debug(rdebug)
    );

assign o_debug = {tdebug, i_Transmit, o_Tx_Active, o_Tx_Done, o_Tx, Tx_send, tx_state[2:0]};

endmodule
