`timescale 1ns / 1ps
//
// this tests the serial communications in the BASYS3 board
//
// push center pushbutton (center of cross) to transmit latched value (ADC or sw[7L)
// push upper pushbutton to display version on LED display
// push left pushbutton to latch adc value (need to do this before transmitting over serial port)
// push right pushbutton to reset registers
// pubh lower pushbutton to latch switches to send over serial port (for testing, try 00110000 should see '0' arrive)
//
// uses XADC.   
//
// this version sends 2 bytes every time you push btnC
//
module top(
        input clk,
        input btnC,         // for serial IO start
        input btnL,         // for triggering ADC latching
        input btnU,         // to display version number on LED display
        input btnR,         // for reset
        input btnD,         // for serial IO sending what's on the 8 lower switches
        output RsTx,        // uart Transmit
        input RsRx,         // uart Receive
        input adc_n, adc_p, // VCAUX 6, P and N
        output [15:0] led,  // 16 onboard LEDs above the switches
        input [7:0] sw,     // 8 lower switches
        output [6:0] segment,   // 4 digit LED display
        output dp,              // "." next to each LED digit
        output [3:0] digit,     // specifies which digit to display
        output [7:0] JA,        // headers for looking at signals on logic analyzers
        output [7:0] JB,
        output [7:0] JC
    );
parameter BAUD_RATE=1000000;
parameter CLOCK_RATE=50000000;    
parameter CLOCK_DIVIDER = 'd50;  // 50MHz/1Mbaud
parameter VERSION = 'h3008;
//
// CLOCK SECTION HERE....  clock = 327us and clk20 = 20ns
//
wire clock2, clk2;
ClkSynth synth (.clock_in(clk), .clock_slow(clock2), .clk20(clk2) );
wire clock_slow, clk20;
BUFG slowclk (.I(clock2),.O(clock_slow));
BUFG clk20buf (.I(clk2),.O(clk20));
//
// debounce here.   note the use of a 2x clock so that we don't miss anything!
//
wire reset = btnR;
//
wire pbC_pushed, pbC_down, pbC_up;
PB_Debouncer debounceC (
    .i_clk(clk20),
    .i_reset(reset),
    .i_PB(btnC),
    .o_PB_state(pbC_pushed),
    .o_PB_down(pbC_down),
    .o_PB_up(pbC_up),
    .o_debug()
    );
//
wire pbU_pushed, pbU_down, pbU_up;
wire [7:0] pbdebug;
PB_Debouncer debounceU (
    .i_clk(clk20),
    .i_reset(reset),
    .i_PB(btnU),
    .o_PB_state(pbU_pushed),
    .o_PB_down(pbU_down),
    .o_PB_up(pbU_up),
    .o_debug(pbdebug)
    );
//
wire pbD_pushed, pbD_down, pbD_up;
PB_Debouncer debounceD (
    .i_clk(clk20),
    .i_reset(reset),
    .i_PB(btnD),
    .o_PB_state(pbD_pushed),
    .o_PB_down(pbD_down),
    .o_PB_up(pbD_up),
    .o_debug()
);
//
wire pbL_pushed, pbL_down, pbL_up;
PB_Debouncer debounceA (
    .i_clk(clk20),
    .i_reset(reset),
    .i_PB(btnL),
    .o_PB_state(pbL_pushed),
    .o_PB_down(pbL_down),
    .o_PB_up(pbL_up),
    .o_debug()
    );
//

wire rx_dv;
wire [7:0] rx_data;
wire [15:0] debugit;
//
// XADC instantiation
//
wire [6:0] daddr_in = 7'h16;
wire adc_ready;
wire eos_out, isbusy, alarm, adc_data_ready;
wire [15:0] adc_data, data_in;
assign data_in = 16'h0;
wire [4:0] channel_out;
reg [15:0] r_adc_data;
localparam [1:0] WAIT=0, LATCH=1, WAIT_END=2;
reg [1:0] adc_state;
reg [15:0] latched_adc;
//
// wait for push button to digitize.  run FSM at 50MHz (mainly so we can see it on the Saleae!)
//
always @ (posedge clk20) begin
    if (reset) r_adc_data <= 16'h0;
    else if (adc_data_ready) r_adc_data <= adc_data;
    end
//
// FSM for latching adc value
//
always @ (posedge clk20 or posedge reset) begin
    if (reset) begin
        adc_state <= WAIT;
        latched_adc <= 0;
    end
    else case (adc_state)
        WAIT: begin
            //
            // watch for btnL or btnD to signal we want to latch something
            //
            if (pbL_pushed || pbD_pushed) adc_state <= LATCH;
            else adc_state <= WAIT;
            end
        LATCH: begin
            //
            // latch it
            //
            if (pbL_pushed) latched_adc <= r_adc_data;
            if (pbD_pushed) latched_adc <= {sw[7:0],8'hA5};
            adc_state <= WAIT_END;
            end
        WAIT_END: begin
            //
            // wait for the button to no longer be pushed
            //
            if (pbL_pushed || pbD_pushed) adc_state <= WAIT_END;
            else adc_state <= WAIT;
            end
        default: begin
            adc_state <= WAIT;
            end
    endcase
    end
//            
//  input clock is 50MHz (specify in the Wizard)
//
xadc_wiz_0 XADC_INST (
      .daddr_in(daddr_in),
      .dclk_in(clk20),
      .den_in(adc_ready),
      .di_in(data_in),
      .dwe_in(1'b0),
      .vauxp6(adc_p),
      .vauxn6(adc_n),
      .busy_out(isbusy),
      .channel_out(channel_out),
      .do_out(adc_data),
      .drdy_out(adc_data_ready),
      .eoc_out(adc_ready),
      .eos_out(eos_out),
      .alarm_out(alarm),
      .vp_in(1'b0),
      .vn_in(1'b0)
      );
//
// send the 16 bits out the serial port.  drive it at 50MHz so that our 100MHz LA can see it!
//
wire [15:0] tx_data = latched_adc[15:0];
wire tx_done, tx_ready;
SerialIO2  # (.CLKS_PER_BIT(CLOCK_DIVIDER)) serial (
    .i_Clock(clk20),
    .i_Reset(reset),
    // transmitter:
    .o_Tx(RsTx),
    .i_Transmit(pbC_down),
    .i_Tx_Byte2(tx_data),
    .o_Tx_Active(tx_ready),
    .o_Tx_Done(tx_done),
    // receiver:
    .i_Rx(RsRx),
    .o_Rx_Byte(rx_data),
    .o_Rx_DV(rx_dv),
    .o_debug(debugit)
    );
//
// drive the LEDs
//
assign led = {~tx_ready,3'b000,r_adc_data[15:4]};
//
// drive the 4 digit LED display
//
wire [15:0] display_this = pbU_pushed ? VERSION : {4'h0,latched_adc};
display4 DISPLAY (
    .clk100(clk),
    .number(display_this),
    .digit(digit),
    .segments(segment),
    .period(dp)
    );
//
// output to JA, JB, and JC header for logic analyzer display
//
assign JA = {r_adc_data[15:8]};
//assign JB = {clk20,adc_data_ready,adc_ready,isbusy,r_adc_data[7:4]};
//assign JC = {adc_data[15:8]};
assign JB = {clk20,debugit[14:8]};
assign JC = debugit[7:0];
//assign JA = {rx_data[5:0],rx_dv,RsRx};
endmodule
